/// <reference types="sequelize" />
import * as Sequelize from 'sequelize';
import { Spec, SpecType, Model } from 'pretur.spec';
import { ProviderPool } from './pool';
import { TableCreationHook, TableDestructionHook, DatabaseAfterCreationHook, DatabaseAfterDestructionHook } from './buildDatabase';
export declare type DatabaseInstance<T extends SpecType> = Sequelize.Instance<Partial<Model<T>>> & Partial<Model<T>>;
export declare type DatabaseModel<T extends SpecType> = Sequelize.Model<DatabaseInstance<T>, Partial<Model<T>>>;
export interface UninitializedDatabaseModel<T extends SpecType> {
    database: DatabaseModel<T>;
    creationHook?: TableCreationHook;
    destructionHook?: TableDestructionHook;
    afterDatabaseCreationHook?: DatabaseAfterCreationHook;
    afterDatabaseDestructionHook?: DatabaseAfterDestructionHook;
    initialize(pool: ProviderPool): void;
}
export interface BuildDatabaseOptions<T extends SpecType> {
    attributeToFieldMap?: {
        [P in keyof T['fields']]?: string;
    };
    tableName?: string;
    creationHook?: TableCreationHook;
    destructionHook?: TableDestructionHook;
    afterDatabaseCreationHook?: DatabaseAfterCreationHook;
    afterDatabaseDestructionHook?: DatabaseAfterDestructionHook;
}
export declare function buildDatabaseModel<T extends SpecType>(spec: Spec<T>, sequelize: Sequelize.Sequelize, options?: BuildDatabaseOptions<T>): UninitializedDatabaseModel<T>;
