import { Spec, SpecType, Model } from 'pretur.spec';
import { Query } from 'pretur.sync';
import { Resolver, UnitializedResolver, ResolveResult } from './resolver';
import { Synchronizer, UnitializedSynchronizer, SyncResult } from './synchronizer';
import { ProviderPool, Transaction } from './pool';
import { UninitializedDatabaseModel, DatabaseModel } from './database';
import { TableCreationHook, TableDestructionHook, DatabaseAfterCreationHook, DatabaseAfterDestructionHook } from './buildDatabase';
export declare type AliasModelMap<T extends SpecType> = {
    [P in keyof T['records'] | keyof T['sets']]: string;
};
export declare type AliasKeyMap<T extends SpecType> = {
    [P in keyof T['records'] | keyof T['sets']]: keyof T['fields'];
};
export interface ProviderMetadata<T extends SpecType> {
    primaryKeys: (keyof T['fields'])[];
    creationHook?: TableCreationHook;
    destructionHook?: TableDestructionHook;
    afterDatabaseCreationHook?: DatabaseAfterCreationHook;
    afterDatabaseDestructionHook?: DatabaseAfterDestructionHook;
    resolver?: Resolver<T>;
    synchronizer?: Synchronizer<T>;
    aliasModelMap: AliasModelMap<T>;
    aliasKeyMap: AliasKeyMap<T>;
    allowedAttributes: (keyof T['fields'])[];
    mutableAttributes: (keyof T['fields'])[];
    defaultOrder: [keyof T['fields'], 'ASC' | 'DESC'];
    sanitizeAttributes(attributes?: (keyof T['fields'])[] | keyof T['fields']): (keyof T['fields'])[];
    initialize(pool: ProviderPool): void;
}
export interface Provider<T extends SpecType> {
    spec: Spec<T>;
    name: string;
    metadata: ProviderMetadata<T>;
    database: DatabaseModel<T>;
    select(tr: Transaction, query?: Query<T>, context?: any): Promise<ResolveResult<T>>;
    insert(tr: Transaction, data: Partial<Model<T>>, context?: any): Promise<SyncResult<T>>;
    update(tr: Transaction, data: Partial<T['fields']>, context?: any): Promise<SyncResult<T>>;
    remove(tr: Transaction, data: Partial<T['fields']>, context?: any): Promise<SyncResult<T>>;
}
export interface BuildProviderOptions<T extends SpecType> {
    database?: UninitializedDatabaseModel<T>;
    resolver?: UnitializedResolver<T>;
    synchronizer?: UnitializedSynchronizer<T>;
    defaultOrder?: [keyof T['fields'], 'ASC' | 'DESC'];
    allowedAttributes?: (keyof T['fields'])[];
    allowedMutableAttributes?: (keyof T['fields'])[];
}
export declare function buildProvider<T extends SpecType>(spec: Spec<T>, options?: BuildProviderOptions<T>): Provider<T>;
