import { Bundle } from 'pretur.i18n';
import { Spec, SpecType, Model } from 'pretur.spec';
import { ProviderPool, Transaction } from './pool';
import { InsertMutateRequest, UpdateMutateRequest, RemoveMutateRequest, MutateRequest } from 'pretur.sync';
export interface SyncResult<T extends SpecType> {
    errors: Bundle[];
    generatedIds?: Partial<T['fields']>;
}
export interface Insert<T extends SpecType> {
    (transaction: Transaction, item: InsertMutateRequest<T>, context: any): Promise<SyncResult<T>>;
}
export interface Update<T extends SpecType> {
    (transaction: Transaction, item: UpdateMutateRequest<T>, context: any): Promise<SyncResult<T>>;
}
export interface Remove<T extends SpecType> {
    (transaction: Transaction, item: RemoveMutateRequest<T>, context: any): Promise<SyncResult<T>>;
}
export interface Synchronizer<T extends SpecType> {
    (transaction: Transaction, item: MutateRequest<T>, context: any): Promise<SyncResult<T>>;
}
export interface UnitializedSynchronizer<T extends SpecType> {
    synchronizer: Synchronizer<T>;
    initialize(pool: ProviderPool): void;
}
export interface InsertSyncInterceptor<T extends SpecType> {
    (insert: (data: Partial<Model<T>>) => Promise<SyncResult<T>>, transaction: Transaction, data: Partial<Model<T>>, context: any): Promise<SyncResult<T>>;
}
export interface UpdateSyncInterceptor<T extends SpecType> {
    (update: (data: Partial<T['fields']>) => Promise<SyncResult<T>>, transaction: Transaction, data: Partial<T['fields']>, context: any): Promise<SyncResult<T>>;
}
export interface RemoveSyncInterceptor<T extends SpecType> {
    (remove: (identifiers: Partial<T['fields']>) => Promise<SyncResult<T>>, transaction: Transaction, identifiers: Partial<T['fields']>, context: any): Promise<SyncResult<T>>;
}
export interface BuildSynchronizerOptions<T extends SpecType> {
    insertInterceptor?: InsertSyncInterceptor<T>;
    updateInterceptor?: UpdateSyncInterceptor<T>;
    removeInterceptor?: RemoveSyncInterceptor<T>;
}
export declare function buildSynchronizer<T extends SpecType>(spec: Spec<T>, options?: BuildSynchronizerOptions<T>): UnitializedSynchronizer<T>;
